#ifndef HIDSCOPE_H_INCLUDED
#define HIDSCOPE_H_INCLUDED

#include "..\miniscope\device\MiniscopeDeviceCapabilities.h"
#include <vector>
#include <windef.h>
#include "singleton.h"

class HidScope : public CSingleton<HidScope>
{
private:
    friend class CSingleton<HidScope>;
	HidScope();
	~HidScope();

	enum { VENDOR_ID = 0x04D8 };
	enum { PRODUCT_ID = 0x003F };

	bool bDeviceOpened;
	bool bStopRequest;
    unsigned int sample_count;  // counter for samples sent to user
    std::vector<short> samples;
    int sampling_period;        // ms
    int sampling_frame;
    HANDLE Thread, ThreadTimer, hEvent;

	typedef void (*CallbackLog)(char*);
	typedef void (*CallbackData)(short*, int);
	typedef void (*CallbackConnect)(int, char*);
	typedef void (*CallbackTrigger)(void);
	void log(char *lpData, ...);

    static VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime);
    friend int CallbackDeviceState(std::string path, bool attached);
    friend DWORD WINAPI ThreadProc(LPVOID data);
    friend DWORD WINAPI ThreadTimerProc(LPVOID data);

public:

	int Connect(void);
	int Disconnect(void);
	CallbackLog callbackLog;
	CallbackData callbackData;
	CallbackConnect callbackConnect;
	CallbackTrigger callbackTrigger;
	void ConnectCb(int state, char* msg) {
		if (callbackConnect)
			callbackConnect(state, msg);
	}
	/** \brief Request manual triggering
		\return 0 on success
	*/
	int ManualTrigger(void);
	/** \param period sampling period in ms
	*/
	int SetSampling (int period) {
	    log("Sampling period set to %d ms.", period);
	    sampling_period = period;
	    return 0;
	}

	int SetBufferSize (int size) {
	    log("Buffer size set to %d ms.", size);
	    sample_count = size;
	    sampling_frame = size;
	    return 0;
	}

	void setOpened(bool opened) {
        bDeviceOpened = opened;
	}

};

#define hidScope HidScope::Instance()

#endif // HIDSCOPE_H_INCLUDED
